﻿Type=Class
Version=2.5
@EndOfDesignText@
'Class Name: PinchZoomAndMove
'Author: Dominex
'Version: 1.3
'B4A Version Used: 2.22
'Last Modified: 02/11/2012
'-------------------------
'Class module
Private Sub Class_Globals
	Private G As Gestures
	Private Touch As List
	Private X0,Y0,X1,Y1,Hypotenuse,Product,posTouchX,posTouchY,vTag,OrigW,OrigH As Int
	Private vZoom,vZoomStart,vZoomMin,vZoomMax,vIncrease As Float
	Private vCanMove,SingleClick,vBringToFront As Boolean
	Private obj As View
	Private vNameEvent As String
End Sub

'Initialize the View.
'<i>Target</i> - View on which to apply the PinchZoomAndMove.
'<i>NameEvent</i> - name of the event.
'<i>ZoomMin</i> - minimum Zoom (eg. 0.5 = 50%).
'<i>ZoomMax</i> - maximum Zoom (eg. 3 = 300%).
'<i>Increase</i> - factor of increase / decrease during the Pinch Zoom.
'<i>Tag</i> - is the Tags View.
'<i>BringToFront</i> - if True active the BringToFront.
Public Sub Initialize (Target As View,NameEvent As String,Tag As Int,ZoomMin As Float,ZoomMax As Float,Increase As Float,BringToFront As Boolean)
	Touch.Initialize
	obj = Target
	OrigH = obj.Height
	OrigW = obj.Width
	vZoom = OrigW
	vZoomStart = vZoom
	vZoomMin = vZoom*ZoomMin
	vZoomMax = vZoom*ZoomMax
	vIncrease = Increase
	vCanMove = True
	vNameEvent = NameEvent
	vTag = Tag
	vBringToFront = BringToFront
	G.SetOnTouchListener(obj,"TouchGestures")
End Sub

'Sets whether the View may translare or not.
'<i>Option</i> - True if the View can translate.
Public Sub SetIfCanMove (Option As Boolean)
	vCanMove = Option
End Sub

'Returns if the View can translate.
Public Sub GetIfCanMove As Boolean
	Return vCanMove
End Sub

'Returns the current Zoom in percentage (eg. 100 = 100%).
Public Sub GetZoom As Float
	Return 100/vZoomStart*vZoom
End Sub

'Sets the zoom.
'<i>Zoom</i> - Zoom in percentage (eg. 100 = 100% = initial size of the View).
Public Sub SetZoom (Zoom As Float)
	ZoomView (Zoom/100*vZoomStart,obj.Width/2,obj.Height/2)
	vZoom = vZoomStart/100*Zoom
End Sub

Private Sub TouchGestures(View As Object, PointerID As Int, Action As Int, X As Float, Y As Float) As Boolean
	Dim conta,xTemp,yTemp,Hypo,NewZoom As Int
	Select Action
		Case G.ACTION_DOWN
			Touch.Add(PointerID)
			SingleClick = True
			If vBringToFront Then obj.BringToFront
			'---Stores the position of View---
			X0 = X+obj.Left
			Y0 = Y+obj.Top
			X1 = obj.Left
			Y1 = obj.Top
		Case G.ACTION_POINTER_DOWN
			Touch.Add(PointerID)
			If Touch.Size = 2 Then 
				xTemp = G.GetX(Touch.Get(0))-G.GetX(Touch.Get(1))
				yTemp = G.GetY(Touch.Get(0))-G.GetY(Touch.Get(1))
				Hypotenuse = Sqrt(Power(xTemp,2)+Power(yTemp,2))/Density
			End If
			SingleClick = False
		Case G.ACTION_MOVE
			If Touch.Size = 1 Then
				'---Move View---
				If vCanMove Then
					xTemp = G.GetX(Touch.Get(0))+obj.Left-X0
					yTemp = G.GetY(Touch.Get(0))+obj.Top-Y0
					obj.Left = X1+xTemp
					obj.Top = Y1+yTemp
				End If
			Else If Touch.Size = 2 Then
				'---Pinch Zoom---
				xTemp = G.GetX(Touch.Get(0))-G.GetX(Touch.Get(1))
				yTemp = G.GetY(Touch.Get(0))-G.GetY(Touch.Get(1))
				Hypo = Sqrt(Power(xTemp,2)+Power(yTemp,2))/Density
				Product = (Hypo-Hypotenuse)*vIncrease
				NewZoom = vZoom+Product
				NewZoom = Min(NewZoom,vZoomMax)
				NewZoom = Max(NewZoom,vZoomMin)
				posTouchX = Min(G.GetX(Touch.Get(0)),G.GetX(Touch.Get(1)))+Abs(xTemp/2)
				posTouchY = Min(G.GetY(Touch.Get(0)),G.GetY(Touch.Get(1)))+Abs(yTemp/2)
				ZoomView (NewZoom,posTouchX,posTouchY) 'Show the new zoom
			End If
		Case G.ACTION_POINTER_UP
			'---Removes touches are no longer present on the screen---
			For conta = 0 To Touch.Size-1
				If Touch.Get(conta) = PointerID Then
					Touch.RemoveAt(conta)
					Exit
				End If
			Next
			If Touch.Size = 1 Then
				X0 = G.GetX(Touch.Get(0))+obj.Left
				Y0 = G.GetY(Touch.Get(0))+obj.Top
				X1 = obj.Left
				Y1 = obj.Top
				'---Save the size of View---
				vZoom = vZoom+Product
				vZoom = Min(vZoom,vZoomMax)
				vZoom = Max(vZoom,vZoomMin)
			End If
		Case G.ACTION_UP
			Touch.Clear
			If Abs(X+obj.Left-X0) < 5 AND Abs(Y+obj.Top-Y0) < 5 AND SingleClick Then
				If SubExists("Main",vNameEvent&"_Click") Then
					Dim lista As List
					lista.Initialize2(Array As Int(X,Y,vTag))
					CallSub2("Main",vNameEvent&"_Click",lista)
				End If
			End If
	End Select
	Return True
End Sub

Private Sub ZoomView (NewZoom As Int,posX As Int,posY As Int)
	Dim vWidth,vHeight As Int
	vWidth = NewZoom
	vHeight = OrigH/OrigW*vWidth
	Dim Left,Top As Int
	Left = obj.Left+posX-(posX/obj.Width*vWidth)
	Top = obj.Top+posY-(posY/obj.Height*vHeight)
	obj.SetLayout(Left,Top,vWidth,vHeight)
End Sub
